
using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Web;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Text.Encodings.Web;
using System.Text.Json;
using Microsoft.Extensions.Configuration;
using Microsoft.AspNetCore.Components;
using System.Threading.Tasks;
using Radzen;

namespace Razen2.Client
{
    public partial class Radzen2Service
    {
        private readonly HttpClient httpClient;
        private readonly Uri baseUri;
        private readonly NavigationManager navigationManager;

        public Radzen2Service(NavigationManager navigationManager, HttpClient httpClient, IConfiguration configuration)
        {
            this.httpClient = httpClient;

            this.navigationManager = navigationManager;
            this.baseUri = new Uri($"{navigationManager.BaseUri}odata/Radzen2/");
        }


        public async System.Threading.Tasks.Task ExportProjectsToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/radzen2/projects/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/radzen2/projects/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async System.Threading.Tasks.Task ExportProjectsToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/radzen2/projects/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/radzen2/projects/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnGetProjects(HttpRequestMessage requestMessage);

        public async Task<Radzen.ODataServiceResult<Razen2.Server.Models.Radzen2.Project>> GetProjects(Query query)
        {
            return await GetProjects(filter:$"{query.Filter}", orderby:$"{query.OrderBy}", top:query.Top, skip:query.Skip, count:query.Top != null && query.Skip != null);
        }

        public async Task<Radzen.ODataServiceResult<Razen2.Server.Models.Radzen2.Project>> GetProjects(string filter = default(string), string orderby = default(string), string expand = default(string), int? top = default(int?), int? skip = default(int?), bool? count = default(bool?), string format = default(string), string select = default(string))
        {
            var uri = new Uri(baseUri, $"Projects");
            uri = Radzen.ODataExtensions.GetODataUri(uri: uri, filter:filter, top:top, skip:skip, orderby:orderby, expand:expand, select:select, count:count);

            var httpRequestMessage = new HttpRequestMessage(HttpMethod.Get, uri);

            OnGetProjects(httpRequestMessage);

            var response = await httpClient.SendAsync(httpRequestMessage);

            return await Radzen.HttpResponseMessageExtensions.ReadAsync<Radzen.ODataServiceResult<Razen2.Server.Models.Radzen2.Project>>(response);
        }

        partial void OnCreateProject(HttpRequestMessage requestMessage);

        public async Task<Razen2.Server.Models.Radzen2.Project> CreateProject(Razen2.Server.Models.Radzen2.Project project = default(Razen2.Server.Models.Radzen2.Project))
        {
            var uri = new Uri(baseUri, $"Projects");

            var httpRequestMessage = new HttpRequestMessage(HttpMethod.Post, uri);

            httpRequestMessage.Content = new StringContent(Radzen.ODataJsonSerializer.Serialize(project), Encoding.UTF8, "application/json");

            OnCreateProject(httpRequestMessage);

            var response = await httpClient.SendAsync(httpRequestMessage);

            return await Radzen.HttpResponseMessageExtensions.ReadAsync<Razen2.Server.Models.Radzen2.Project>(response);
        }

        partial void OnDeleteProject(HttpRequestMessage requestMessage);

        public async Task<HttpResponseMessage> DeleteProject(Guid id = default(Guid))
        {
            var uri = new Uri(baseUri, $"Projects({id})");

            var httpRequestMessage = new HttpRequestMessage(HttpMethod.Delete, uri);

            OnDeleteProject(httpRequestMessage);

            return await httpClient.SendAsync(httpRequestMessage);
        }

        partial void OnGetProjectById(HttpRequestMessage requestMessage);

        public async Task<Razen2.Server.Models.Radzen2.Project> GetProjectById(string expand = default(string), Guid id = default(Guid))
        {
            var uri = new Uri(baseUri, $"Projects({id})");

            uri = Radzen.ODataExtensions.GetODataUri(uri: uri, filter:null, top:null, skip:null, orderby:null, expand:expand, select:null, count:null);

            var httpRequestMessage = new HttpRequestMessage(HttpMethod.Get, uri);

            OnGetProjectById(httpRequestMessage);

            var response = await httpClient.SendAsync(httpRequestMessage);

            return await Radzen.HttpResponseMessageExtensions.ReadAsync<Razen2.Server.Models.Radzen2.Project>(response);
        }

        partial void OnUpdateProject(HttpRequestMessage requestMessage);
        
        public async Task<HttpResponseMessage> UpdateProject(Guid id = default(Guid), Razen2.Server.Models.Radzen2.Project project = default(Razen2.Server.Models.Radzen2.Project))
        {
            var uri = new Uri(baseUri, $"Projects({id})");

            var httpRequestMessage = new HttpRequestMessage(HttpMethod.Patch, uri);


            httpRequestMessage.Content = new StringContent(Radzen.ODataJsonSerializer.Serialize(project), Encoding.UTF8, "application/json");

            OnUpdateProject(httpRequestMessage);

            return await httpClient.SendAsync(httpRequestMessage);
        }
    }
}