using System;
using System.Net;
using System.Data;
using System.Linq;
using Microsoft.Data.SqlClient;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;

using Microsoft.AspNetCore.OData.Query;
using Microsoft.AspNetCore.OData.Routing.Controllers;
using Microsoft.AspNetCore.OData.Results;
using Microsoft.AspNetCore.OData.Deltas;
using Microsoft.AspNetCore.OData.Formatter;

using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Internal;

namespace Razen2.Server.Controllers.Radzen2
{
    [Route("odata/Radzen2/Projects")]
    public partial class ProjectsController : ODataController
    {
        private Razen2.Server.Data.Radzen2Context context;

        public ProjectsController(Razen2.Server.Data.Radzen2Context context)
        {
            this.context = context;
        }

    
        [HttpGet]
        [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
        public IEnumerable<Razen2.Server.Models.Radzen2.Project> GetProjects()
        {
            var items = this.context.Projects.AsQueryable<Razen2.Server.Models.Radzen2.Project>();
            this.OnProjectsRead(ref items);

            return items;
        }

        partial void OnProjectsRead(ref IQueryable<Razen2.Server.Models.Radzen2.Project> items);

        partial void OnProjectGet(ref SingleResult<Razen2.Server.Models.Radzen2.Project> item);

        [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
        [HttpGet("/odata/Radzen2/Projects(Id={Id})")]
        public SingleResult<Razen2.Server.Models.Radzen2.Project> GetProject(Guid key)
        {
            var items = this.context.Projects.Where(i => i.Id == key);
            var result = SingleResult.Create(items);

            OnProjectGet(ref result);

            return result;
        }
        partial void OnProjectDeleted(Razen2.Server.Models.Radzen2.Project item);
        partial void OnAfterProjectDeleted(Razen2.Server.Models.Radzen2.Project item);

        [HttpDelete("/odata/Radzen2/Projects(Id={Id})")]
        public IActionResult DeleteProject(Guid key)
        {
            try
            {
                if (!ModelState.IsValid)
                {
                    return BadRequest(ModelState);
                }


                var item = this.context.Projects
                    .Where(i => i.Id == key)
                    .FirstOrDefault();

                if (item == null)
                {
                    return BadRequest();
                }
                this.OnProjectDeleted(item);
                this.context.Projects.Remove(item);
                this.context.SaveChanges();
                this.OnAfterProjectDeleted(item);

                return new NoContentResult();

            }
            catch(Exception ex)
            {
                ModelState.AddModelError("", ex.Message);
                return BadRequest(ModelState);
            }
        }

        partial void OnProjectUpdated(Razen2.Server.Models.Radzen2.Project item);
        partial void OnAfterProjectUpdated(Razen2.Server.Models.Radzen2.Project item);

        [HttpPut("/odata/Radzen2/Projects(Id={Id})")]
        [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
        public IActionResult PutProject(Guid key, [FromBody]Razen2.Server.Models.Radzen2.Project item)
        {
            try
            {
                if(!ModelState.IsValid)
                {
                    return BadRequest(ModelState);
                }

                if (item == null || (item.Id != key))
                {
                    return BadRequest();
                }
                this.OnProjectUpdated(item);
                this.context.Projects.Update(item);
                this.context.SaveChanges();

                var itemToReturn = this.context.Projects.Where(i => i.Id == key);
                ;
                this.OnAfterProjectUpdated(item);
                return new ObjectResult(SingleResult.Create(itemToReturn));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("", ex.Message);
                return BadRequest(ModelState);
            }
        }

        [HttpPatch("/odata/Radzen2/Projects(Id={Id})")]
        [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
        public IActionResult PatchProject(Guid key, [FromBody]Delta<Razen2.Server.Models.Radzen2.Project> patch)
        {
            try
            {
                if(!ModelState.IsValid)
                {
                    return BadRequest(ModelState);
                }

                var item = this.context.Projects.Where(i => i.Id == key).FirstOrDefault();

                if (item == null)
                {
                    return BadRequest();
                }
                patch.Patch(item);

                this.OnProjectUpdated(item);
                this.context.Projects.Update(item);
                this.context.SaveChanges();

                var itemToReturn = this.context.Projects.Where(i => i.Id == key);
                ;
                return new ObjectResult(SingleResult.Create(itemToReturn));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("", ex.Message);
                return BadRequest(ModelState);
            }
        }

        partial void OnProjectCreated(Razen2.Server.Models.Radzen2.Project item);
        partial void OnAfterProjectCreated(Razen2.Server.Models.Radzen2.Project item);

        [HttpPost]
        [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
        public IActionResult Post([FromBody] Razen2.Server.Models.Radzen2.Project item)
        {
            try
            {
                if(!ModelState.IsValid)
                {
                    return BadRequest(ModelState);
                }

                if (item == null)
                {
                    return BadRequest();
                }

                this.OnProjectCreated(item);
                this.context.Projects.Add(item);
                this.context.SaveChanges();

                var itemToReturn = this.context.Projects.Where(i => i.Id == item.Id);

                ;

                this.OnAfterProjectCreated(item);

                return new ObjectResult(SingleResult.Create(itemToReturn))
                {
                    StatusCode = 201
                };
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("", ex.Message);
                return BadRequest(ModelState);
            }
        }
    }
}
